from threading import Event
try:
	import cv2
	import numpy as np
except:
	print('Install OpenCV with "pip3 install opencv-python"')
	print('Press Enter key for exit')
	input_data = input()
	quit()
try:
	from record3d import Record3DStream
except:
	print('Install Record3D with "pip3 install record3d"')
	print('Press Enter key for exit')
	input_data = input()
	quit()
try:
	import pyvirtualcam
except:
	print('Install pyvirtualcam with "pip install pyvirtualcam"')
	print(' and Download and Install obs-virtualcam from https://github.com/Fenrirthviti/obs-virtual-cam/releases')
	print('Press Enter key for exit')
	input_data = input()
	quit()

class DemoApp:
    def __init__(self):
        self.event = Event()
        self.session = None

    def on_new_frame(self):
        """
        This method is called from non-main thread, therefore cannot be used for presenting UI.
        """
        self.event.set()  # Notify the main thread to stop waiting and process new frame.

    def on_stream_stopped(self):
        print('Stream stopped')

    def connect_to_device(self, dev_idx):
        print('Searching for devices')
        devs = Record3DStream.get_connected_devices()
        print('{} device(s) found'.format(len(devs)))
        for dev in devs:
            print('\tID: {}\n\tUDID: {}\n'.format(dev.product_id, dev.udid))

        if len(devs) <= dev_idx:
            print('Cannot connect to iPhone')
            print('Press Enter key for exit')
            input_data = input()
            raise RuntimeError('Cannot connect to device #{}, try different index.'
                               .format(dev_idx))

        dev = devs[dev_idx]
        self.session = Record3DStream()
        self.session.on_new_frame = self.on_new_frame
        self.session.on_stream_stopped = self.on_stream_stopped
        self.session.connect(dev)  # Initiate connection and start capturing

    def get_intrinsic_mat_from_coeffs(self, coeffs):
        return np.array([[coeffs.fx,         0, coeffs.tx],
                         [        0, coeffs.fy, coeffs.ty],
                         [        0,         0,         1]])


    def start_processing_stream(self):
        with pyvirtualcam.Camera(width=1280, height=480, fps=20, print_fps=False,delay=0) as vcam:
            while True:
                self.event.wait()  # Wait for new frame to arrive

                # Copy the newly arrived RGBD frame
                depth = self.session.get_depth_frame()
                rgb = self.session.get_rgb_frame()
                intrinsic_mat = self.get_intrinsic_mat_from_coeffs(self.session.get_intrinsic_mat())
                # You can now e.g. create point cloud by projecting the depth map using the intrinsic matrix.

                # Postprocess it
                are_truedepth_camera_data_being_streamed = depth.shape[0] == 640
                if are_truedepth_camera_data_being_streamed:
                    depth = cv2.flip(depth, 1)
                    rgb = cv2.flip(rgb, 1)

                depth = cv2.cvtColor(depth, cv2.COLOR_GRAY2RGB)
                rgb = np.array(rgb)
                rgb = rgb.astype(np.uint8)
                depth = np.array(depth)
                depth = np.minimum(255, (depth) * 255)
                depth = depth.astype(np.uint8)
                depth = np.where(depth==0, 255, depth)
                depth = cv2.rotate(depth, cv2.ROTATE_90_COUNTERCLOCKWISE)
                rgb = cv2.rotate(rgb, cv2.ROTATE_90_COUNTERCLOCKWISE)
                frame = cv2.hconcat([rgb, depth])
                if frame.shape[0] != 480:
                    frame = cv2.resize(frame, (1280, 480))
                frame = cv2.cvtColor(frame, cv2.COLOR_RGB2RGBA)
                # Send RGB+Depth Stream to OBS-Camera
                vcam.send(frame)

                # Show the RGBD Stream
                frame = cv2.cvtColor(frame, cv2.COLOR_RGBA2BGR)
                cv2.imshow('OBS-Camera[1280x480] - Q key to close', frame)
                # cv2.imshow('Depth', depth2)
                key = cv2.waitKey(1)
                if key == ord('q'):
                    break
                self.event.clear()


if __name__ == '__main__':
    app = DemoApp()
    app.connect_to_device(dev_idx=0)
    app.start_processing_stream()
